@@ -6,7 +6,7 @@ use IO::Socket::INET;
 use Storable qw(thaw nfreeze);
 use RPC::Object::Common;
 
-our $VERSION = '0.04';
+our $VERSION = '0.10';
 $VERSION = eval $VERSION;
 
 sub new {
@@ -22,6 +22,12 @@ sub new {
     return bless $self, $class;
 }
 
+sub get_instance {
+    my $class = shift;
+    my $url = shift;
+    return $class->new($url, '_get_blessed_instance', 'RPC::Object::Broker', @_);
+}
+
 sub AUTOLOAD {
     my $self = shift;
     my $name = (split '::', our $AUTOLOAD)[-1];
@@ -45,16 +51,25 @@ sub AUTOLOAD {
 sub _invoke {
     my $host = shift;
     my $port = shift;
-    my $sock = IO::Socket::INET->new(PeerAddr => $host,
-                                     PeerPort => $port,
-                                     Proto => 'tcp',
-                                     Type => SOCK_STREAM,
-                                    );
-    binmode $sock;
+    my $sock;
+    my $connected;
+    while (!$connected) {
+        eval {
+            $sock = IO::Socket::INET->new(PeerAddr => $host,
+                                          PeerPort => $port,
+                                          Proto => 'tcp',
+                                          Type => SOCK_STREAM,
+                                         );
+            binmode $sock;
+        };
+        $connected = !$@;
+        sleep 1 unless $connected;
+    }
     print {$sock} nfreeze([wantarray ? WANT_LIST : WANT_SCALAR, @_]);
     $sock->shutdown(1);
     my $res = do { local $/; <$sock> };
     $sock->close();
+
     my ($stat, @ret) = @{thaw($res)};
     if ($stat eq RESPONSE_ERROR) {
         die @ret;
@@ -80,7 +95,7 @@ RPC::Object - A lightweight implementation for remote procedure calls
 B<On server>
 
   use RPC::Object::Broker;
-  $b = $RPC::Object::Broker->new($port);
+  $b = $RPC::Object::Broker->get_instance($port, @preload_modules);
   $b->start();
 
 B<On client>
@@ -90,7 +105,41 @@ B<On client>
   my $ans1 = $o->method_b($arg1, $arg2);
   my @ans2 = $o->method_c($arg3, $arg4);
 
-Please see more examples in the C<eg/> directory.
+  # or access the global instance
+  $o2 = RPC::Object->get_instance("$host:$port", 'method_x', 'TestModule2');
+
+B<TestModule>
+
+  package TestModule;
+  use threads;
+  ...
+  sub method_a {
+      my $class = shift;
+      my $self : shared;
+      ...
+      return bless $self, $class;
+  }
+
+  sub method_b {
+      ...
+  }
+  ...
+
+B<TestModule2>
+
+  package TestModule2;
+  use threads;
+  ...
+  {
+     my $instance : share;
+     sub method_x {
+         ...
+         return $instance;
+     }
+  }
+  ...
+
+Please see more examples in the test scripts.
 
 =head1 DESCRIPTON
 
@@ -100,6 +149,23 @@ modules, e.g. IO and Storable.
 
 Other approaches like SOAP or XML-RPC are too heavy for simple tasks.
 
+B<Thread awareness>
+
+All modules and objects invoked by C<RPC::Object> should aware the multi-threaded envrionment.
+
+B<Constructor and Destructor>
+
+The Module could name its constructor any meaningful name. it do not have to be C<new>, or C<create>, etc...
+
+To release an object, use C<RPC::Object::rpc_object_release>, e.g.
+
+  $o = RPC::Object->new("$host:$port", 'method_a', 'TestModule');
+  $o->rpc_object_release();
+
+B<Global instance>
+
+There are two ways to allocate and access global instances: a) use the preload module list to load the module at server side, and use the C<RPC::Object::new()> method; b) use the C<RPC::Object::get_instance()> method. The performance of method a may be better as modules are only loaded once.
+
 =head1 AUTHORS
 
 Jianyuan Wu <jwu@cpan.org>
